Array.prototype.trimDestructive=function(){for(var a=0;a<this.length;++a)if(this[a]){this.splice(0,a);break}this[0]||this.splice(0,this.length);for(a=this.length-1;0<=a;--a)if(this[a]){this.splice(a+1,this.length-a-1);break}return this};
Object.clone=function(a){return"object"===typeof a||"function"===typeof a&&null!==a?JSON.parse(JSON.stringify(a)):a};

EdvalAPI = (function ()
{
    function parseGeneralSubLine(subLine)
    {
        subLine = subLine.trim();
        var result = {params: {}};
        while (subLine.length)
        {
            var match = subLine.match(/^([A-Z0-9]+)=([^" ]+)/);
            if (!match)
                match = subLine.match(/^([A-Z0-9]+)="([^"]+)"/);
            if (match)
            {
                result.params[match[1]] = match[2];
                subLine = subLine.slice(match[0].length + 1);
            }
            else
            {
                result.unparsed = subLine;
                break;
            }
        }
        return result;
    }
    function parseGeneralLine(line)
    {
        line = line.trim();
        var paramSplitIndex = line.indexOf(" ");
        var result = {label: line.slice(0, paramSplitIndex).trim()};
        $.extend(result, parseGeneralSubLine(line.slice(paramSplitIndex + 1)));
        return result;
    }

    var EdvalAPI = function (rawEdvalData)
    {
        var rawEdvalLines = rawEdvalData.split("\n");
        var sections = {};
        for (var l = rawEdvalLines.length - 1; l > 0; --l)
            if (rawEdvalLines[l].match(/^~+$/))
            {
                sections[this.toId(rawEdvalLines[l - 1])] = rawEdvalLines.splice(l + 1, rawEdvalLines.length - l - 1).trimDestructive();
                rawEdvalLines.splice(-2, 2);
                --l;
            }

        this.periods = new EdvalAPI_Periods(this, sections["periods"]);
        this.classes = new EdvalAPI_Classes(this, sections["classes"]);
        this.rooms = new EdvalAPI_Rooms(this, sections["rooms"]);
        this.grades =
        {
            "7": new EdvalAPI_Grade(this, sections["linesfor7"]),
            "8": new EdvalAPI_Grade(this, sections["linesfor8"]),
            "9": new EdvalAPI_Grade(this, sections["linesfor9"]),
            "10": new EdvalAPI_Grade(this, sections["linesfor10"]),
            "11": new EdvalAPI_Grade(this, sections["linesfor11"]),
            "12": new EdvalAPI_Grade(this, sections["linesfor12"])
        };
        this.teachers = new EdvalAPI_Teachers(this, sections["teachers"]);
    };
    EdvalAPI.prototype =
    {
        toId: function (string) { return string.toLowerCase().replace(/[^a-z0-9._]/g, ""); },
        getPersonInfo: function (id)
        {
            for (var g in this.grades)
            {
                var studentInfo = this.grades[g].getStudentInfo(id);
                if (studentInfo)
                    return $.extend(studentInfo, {type: "student", grade: g});
            }
            var teacherInfo = this.teachers.getTeacherInfo(id);
            if (teacherInfo)
                return $.extend(teacherInfo, {type: "teacher"});
            return null;
        },
        searchPeople: function (query)
        {
            var results = [];
            for (var g in this.grades)
            {
                var gradeResults = this.grades[g].search(query);
                for (var r = 0; r < gradeResults.length; ++r)
                    $.extend(gradeResults[r], {type: "student", grade: g});
                Array.prototype.push.apply(results, gradeResults);
            }
            var teacherResults = this.teachers.search(query);
            for (var r = 0; r < teacherResults.length; ++r)
                $.extend(teacherResults[r], {type: "teacher"});
            Array.prototype.push.apply(results, teacherResults);
            return results;
        },
        buildTimetable: function (searchResult)
        {
            if (typeof searchResult === "string")
            {
                var searchResults = this.searchPeople(searchResult);
                if (searchResults.length !== 1)
                    return null;
                searchResult = searchResults[0];
            }

            if (searchResult.type === "student")
                return this.grades[searchResult.grade].buildTimetable(searchResult.id);
            if (searchResult.type === "teacher")
                return this.teachers.buildTimetable(searchResult.id);
            return null;
        }
    };

    var EdvalAPI_Periods = function (parent, sectionLines)
    {
        this.sections = {};
        for (var l = sectionLines.length - 1; l >= 0; --l)
        {
            var match = sectionLines[l].match(/^\[([a-zA-Z]+)\]$/);
            if (match)
            {
                this.sections[parent.toId(match[1])] = sectionLines.splice(l + 1, sectionLines.length - l - 1).trimDestructive();
                sectionLines.pop();
            }
        }

        this.days = this.sections["days"].splice(0, this.sections["days"].length);
        for (var d = 0; d < this.days.length; ++d)
            this.days[d] = parent.toId(parseGeneralLine(this.days[d]).label);
        this.periods = this.sections["periods"].splice(0, this.sections["periods"].length);
        for (var p = 0; p < this.periods.length; ++p)
            this.periods[p] = parent.toId(parseGeneralLine(this.periods[p]).label);
        this.bellTimes = {};
        this.dayPeriods = {};
        for (var d = 0; d < this.sections["dps"].length; ++d)
        {
            var parsed = parseGeneralLine(this.sections["dps"][d]);
            var day = parent.toId(parsed.params["DA"]);
            var period = parent.toId(parsed.params["PE"]);
            if (!this.bellTimes[day])
                this.bellTimes[day] = {};
            this.bellTimes[day][period] = parsed.params["MI"].split("-");
            this.dayPeriods[parent.toId(parsed.label)] = {day: day, period: period};
        }
    };
    EdvalAPI_Periods.prototype =
    {
        getDays: function ()
        {
            return Object.clone(this.days);
        },
        getPeriods: function ()
        {
            return Object.clone(this.periods);
        },
        getBellTimesForDay: function (day)
        {
            return Object.clone(this.bellTimes[day]);
        },
        getDayAndPeriodFromDayPeriod: function (dayPeriod)
        {
            return Object.clone(this.dayPeriods[dayPeriod]);
        }
    };

    var EdvalAPI_Classes = function (parent, sectionLines)
    {
        this.classes = {};
        while (sectionLines.length)
        {
            var occurrences = sectionLines.pop().trim().match(/\[[^\]]+\]/g);
            for (var o = 0; o < occurrences.length; ++o)
                occurrences[o] = parseGeneralSubLine(occurrences[o].substr(1, occurrences[o].length - 2));
            var general = parseGeneralLine(sectionLines.pop());

            var classInfo = {};
            classInfo.grade = general.params["YR"];
            classInfo.name = general.params["SU"];
            classInfo.faculty = general.params["FA"];
            classInfo.occurrences = [];
            for (var o = 0; o < occurrences.length; ++o)
            {
                var occurrence = {};
                occurrence.teacherId = occurrences[o].params["TE"] ? parent.toId(occurrences[o].params["TE"]) : "";
                occurrence.roomId = occurrences[o].params["RO"] ? parent.toId(occurrences[o].params["RO"]) : "";
                if (occurrences[o].params["TI"])
                    $.extend(occurrence, parent.periods.getDayAndPeriodFromDayPeriod(parent.toId(occurrences[o].params["TI"])));
                classInfo.occurrences.push(occurrence);
            }
            this.classes[parent.toId(general.label)] = classInfo;
        }
    };
    EdvalAPI_Classes.prototype =
    {
        getClassInfo: function (classId)
        {
            return Object.clone(this.classes[classId]);
        },
        getClasses: function ()
        {
            return Object.clone(this.classes);
        }
    };

    var EdvalAPI_Rooms = function (parent, sectionLines)
    {
        this.rooms = {};
        var rooms = sectionLines.splice(0, sectionLines.length).join("\n").split("\n\n")[0].split("\n");
        while (rooms.length)
        {
            var parsed = parseGeneralLine(rooms.pop());
            this.rooms[parent.toId(parsed.label)] = {name: parsed.params["NB"] ? parsed.params["NB"] : parsed.label, homeRoomOf: parsed.params["HR"] ? parent.toId(parsed.params["HR"]) : ""};
        }
    };
    EdvalAPI_Rooms.prototype =
    {
        getRoomInfo: function (roomId)
        {
            return Object.clone(this.rooms[roomId]);
        },
        getRooms: function ()
        {
            return Object.clone(this.rooms);
        }
    };

    var EdvalAPI_Grade = function (parent, sectionLines)
    {
        this.parent = parent;
        this.sections = {};
        for (var l = sectionLines.length - 1; l >= 0; --l)
        {
            var match = sectionLines[l].match(/^\[([a-zA-Z]+)\]$/);
            if (match)
            {
                this.sections[parent.toId(match[1])] = sectionLines.splice(l + 1, sectionLines.length - l - 1).trimDestructive();
                sectionLines.pop();
            }
        }

        this.students = {};
        while (this.sections["students"].length)
        {
            var parsed = parseGeneralLine(this.sections["students"].pop());
            var student = {};

            var name = parsed.params["LN"].split(", ");
            if (name.length === 1)
                name = parsed.params["LN"].split("  ");
            student.lastName = name[0];
            student.firstName = name[1];

            student.classIds = [];
            for (var p in parsed.params)
                if (p[0] === 'Z')
                    student.classIds.push(parent.toId(parsed.params[p]));
            this.students[parent.toId(parsed.label)] = student;
        }

        this.timetableCache = {};
    };
    EdvalAPI_Grade.prototype =
    {
        getStudentInfo: function (studentId)
        {
            return Object.clone(this.students[studentId]);
        },
        getStudents: function ()
        {
            return Object.clone(this.students);
        },
        buildTimetable: function (studentId)
        {
            var student = this.students[studentId];
            if (!student)
                return null;
            if (this.timetableCache[studentId])
                return Object.clone(this.timetableCache[studentId]);

            var result = {};
            result.id = studentId;
            result.lastName = student.lastName;
            result.firstName = student.firstName;
            result.days = {};
            for (var c = 0; c < student.classIds.length; ++c)
            {
                var classInfo = this.parent.classes.getClassInfo(student.classIds[c]);
                for (var o = 0; o < classInfo.occurrences.length; ++o)
                {
                    var occurrence = classInfo.occurrences[o];
                    if (!result.days[occurrence.day])
                        result.days[occurrence.day] = {};
                    result.days[occurrence.day][occurrence.period] = {classId: student.classIds[c], teacherId: occurrence.teacherId, roomId: occurrence.roomId};
                }
            }
            return Object.clone(this.timetableCache[studentId] = result);
        },
        search: function (query)
        {
            query = this.parent.toId(query);
            var results = [];
            for (var s in this.students)
                if (s.indexOf(query) >= 0 || this.parent.toId(this.students[s].lastName).indexOf(query) >= 0 || this.parent.toId(this.students[s].firstName).indexOf(query) >= 0)
                    results.push($.extend(this.students[s], {id: s, toString: function () { return this.firstName + " " + this.lastName; }}));
            return results;
        }
    };

    var EdvalAPI_Teachers = function (parent, sectionLines)
    {
        this.parent = parent;
        this.teachers = {};
        var teachers = sectionLines.splice(0, sectionLines.length).join("\n").split("\n\n\n")[1].split("\n");
        while (teachers.length)
        {
            var parsed = parseGeneralLine(teachers.pop());
            var teacher = {};

            var name = parsed.params["LN"].match(/^([A-Z\- ]+),? ([A-Za-z\- ]+)$/);
            if (name)
            {
                teacher.lastName = name[1];
                teacher.firstName = name[2];
            }
            else
            {
                teacher.lastName = "";
                teacher.firstName = parsed.params["LN"];
            }
            teacher.faculty = parsed.params["FA"];
            this.teachers[parent.toId(parsed.label)] = teacher;
        }
        this.timetableCache = {};
    };
    EdvalAPI_Teachers.prototype =
    {
        getTeacherInfo: function (teacherId)
        {
            return Object.clone(this.teachers[teacherId]);
        },
        getTeachers: function ()
        {
            return Object.clone(this.teachers);
        },
        buildTimetable: function (teacherId)
        {
            var teacher = this.teachers[teacherId];
            if (!teacher)
                return null;
            if (this.timetableCache[teacherId])
                return Object.clone(this.timetableCache[teacherId]);

            var result = {};
            result.id = teacherId;
            result.lastName = teacher.lastName;
            result.firstName = teacher.firstName;
            result.days = {};

            var classes = this.parent.classes.getClasses();
            for (var classId in classes)
                for (var o = 0; o < classes[classId].occurrences.length; ++o)
                {
                    var occurrence = classes[classId].occurrences[o];
                    if (occurrence.teacherId !== teacherId || !occurrence.day)
                        continue;
                    if (!result.days[occurrence.day])
                        result.days[occurrence.day] = {};
                    result.days[occurrence.day][occurrence.period] = {classId: classId, roomId: occurrence.roomId};
                }

            return Object.clone(this.timetableCache[teacherId] = result);
        },
        search: function (query)
        {
            query = this.parent.toId(query);
            var results = [];
            for (var t in this.teachers)
                if (t.indexOf(query) >= 0 || this.parent.toId(this.teachers[t].lastName).indexOf(query) >= 0 || this.parent.toId(this.teachers[t].firstName).indexOf(query) >= 0)
                    results.push($.extend(this.teachers[t], {id: t, toString: function () { return this.firstName + " " + this.lastName; }}));
            return results;
        }
    };

    return EdvalAPI;
})();
