TabManager = function (menuElem, contentElem, parentInfo)
{
    this.parentInfo = parentInfo ? parentInfo : {id: "tab"};
    this.menuElem = $(menuElem);
    this.contentElem = $(contentElem);
    this.tabs = {};
    this.nextId = 0;
};
TabManager.prototype =
{
    toggleMenu: function ()
    {
        var currentMargin = parseInt(this.menuElem.css("margin-left"), 10);
        this.menuElem.animate({"margin-left": currentMargin === 0 ? "-100%" : 0});
    },
    getTab: function (tabId)
    {
        if (tabId.indexOf(this.parentInfo.id) !== 0)
            return null;
        if (this.tabs[tabId])
            return this.tabs[tabId];
        var ourTabId = this.parentInfo.id + "-" + tabId.slice(this.parentInfo.id.length + 1).split("-").shift();
        if (!this.tabs[ourTabId])
            return null;
        return this.tabs[ourTabId].subtabManager.getTab(tabId);
    },
    getTabByName: function (name)
    {
        for (var t in this.tabs)
            if (this.tabs[t].name === name)
                return this.tabs[t];
        return null;
    },
    getActiveTabId: function ()
    {
        return this.menuElem.children(".active").attr("id");
    },
    getInnermostActiveTabId: function ()
    {
        var manager = this;
        var tabId;
        while (tabId = manager.getActiveTabId())
            manager = manager.tabs[tabId].subtabManager;
        if (manager.parentInfo.manager)
            return manager.parentInfo.id;
        return null;
    },
    addTab: function (name, href, autoSelect, events)
    {
        var tab = this.getTabByName(name);
        if (!tab)
        {
            var self = this;
            var id = this.parentInfo.id + "-" + this.nextId++;

            var subtabMenu = $("<ul />");
            var button = $("<li />")
                         .attr({"id": id})
                         .text(name)
                         .click(function (event) { self.selectTab(id); event.stopPropagation(); })
                         .append(subtabMenu);
            if (this.parentInfo.manager)
                button.prepend($("<span />").click(function (event) { self.removeTab(id); event.stopPropagation(); }));
            var frame = $("<iframe />").attr({"src": href, "id": id + "-frame"});

            if (events)
            {
                var eventWrapper = function (callback) { return function (event) { return callback.call(frame[0].contentWindow, event); }; };
                for (var e in events)
                    frame[0].addEventListener(e, eventWrapper(events.load));
            }

            this.menuElem.append(button);
            this.contentElem.append(frame);

            this.tabs[id] = tab =
            {
                manager: this,
                id: id,
                name: name,
                href: href,
                events: events,
                button: button,
                frame: frame,
                subtabManager: new TabManager(subtabMenu, this.contentElem, {id: id, manager: this})
            };
        }
        if (autoSelect || (Object.keys(this.tabs).length === 1 && !this.parentInfo.manager))
            this.selectTab(tab.id);
        return tab;
    },
    removeTab: function (id)
    {
        var tab = this.tabs[id];
        if (!tab)
            return false;

        if (this.getActiveTabId() === id)
        {
            var tabList = Object.keys(this.tabs);
            var tabIndex = tabList.indexOf(id);
            if (tabIndex > 0)
                this.selectTab(tabList[tabIndex - 1]);
            else if (tabIndex + 1 < tabList.length)
                this.selectTab(tabList[tabIndex + 1]);
            else
                this.selectParentTab();
        }

        tab.subtabManager.removeAllTabs();
        delete tab.subtabManager;

        tab.button.remove();
        tab.frame.remove();
        delete this.tabs[id];

        return true;
    },
    removeAllTabs: function ()
    {
        for (var t in this.tabs)
            this.removeTab(this.tabs[t].id);
    },
    selectTab: function (id)
    {
        var tab = this.tabs[id];
        if (!tab)
            return false;
        this.menuElem.find(".active").removeClass("active");
        this.contentElem.children().hide();
        tab.button.addClass("active");
        tab.frame.show();
        return true;
    },
    selectParentTab: function ()
    {
        if (!this.parentInfo.manager)
            return false;
        return this.parentInfo.manager.selectTab(this.parentInfo.id);
    }
};

function updateMenuWidth()
{
    var menu = $("#menu");
    menu.width("20%");
    if (menu.width() * 3 < window.innerHeight)
        menu.width(window.innerHeight / 3);
}
window.addEventListener("resize", updateMenuWidth);
window.addEventListener("load", updateMenuWidth);

function main(edvalData)
{
    edval = new EdvalAPI(edvalData);
    tabs = new TabManager($("#menu"), $("#content"));
    if (!window.java || !java.getTimetableFile)
        tabs.toggleMenu();

    timetables = new Timetables(edval, tabs);
}

window.addEventListener("load", function()
{
    if (!window.java || !java.getTimetableFile)
        $.get("timetable.ett", main, "text");
    else
    {
        var data = java.getTimetableFile();
        if (!data || typeof data !== "string")
            $.ajax(
            {
                url: "http://kota.no-ip.org/timetable/timetable.ett",
                type: "GET",
                dataType: "text",
                error: function(a, b, e) { alert("Could not download timetable: " + e); java.exit(); },
                success: function(data)
                {
                    java.saveTimetableFile(data);
                    main(data);
                }
            });
        else
            main(data);
    }
});
