function buildTimetableOverview(edval, timetable, selectCallback)
{
    var dayToDayNumberMap =
    {
        "mona": 0, "monb": 0,
        "tuea": 1, "tueb": 1,
        "weda": 2, "wedb": 2,
        "thua": 3, "thub": 3,
        "fria": 4, "frib": 4
    };
    var dayNumberToWordMap = { 0: "Monday", 1: "Tuesday", 2: "Wednesday", 3: "Thursday", 4: "Friday" };

    // Preprocess the timetable to make it display properly
    var dayCounts = {};
    for (var d in timetable.days)
    {
        if (!dayCounts[dayToDayNumberMap[d]])
            dayCounts[dayToDayNumberMap[d]] = 0;
        ++dayCounts[dayToDayNumberMap[d]];
    }
    for (var d in dayCounts)
        if (dayCounts[d] === 1)
            for (var n in dayToDayNumberMap)
                if (dayToDayNumberMap[n].toString() === d.toString())
                    if (!timetable.days[n])
                        timetable.days[n] = {};

    var days = edval.periods.getDays();
    var dayList = Object.keys(timetable.days).sort(function (a, b) { return days.indexOf(a) - days.indexOf(b); });
    var periods = edval.periods.getPeriods();
    var periodList = JSON.parse(localStorage.getItem("Timetables::alwaysShownPeriods"));
    for (var d in timetable.days)
        for (var p in timetable.days[d])
            if (periodList.indexOf(p) < 0)
                periodList.push(p);
    periodList.sort(function (a, b) { return periods.indexOf(a) - periods.indexOf(b); });

    var tables = 2; // Don't change this
    var wrapperElem = $("<table />").addClass("timetable-overview");
    for (var table = 1, d = 0; table <= tables; ++table, d += Math.floor(dayList.length / tables))
    {
        for (var p = 0; p < periodList.length; ++p)
        {
            var rowElem = $("<tr />").append($("<th />").addClass("period").text(periodList[p].toUpperCase()));
            for (var d2 = d; d2 < dayList.length * table / tables; ++d2)
            {
                var callbackWrapper = (function (day) { return function () { if (Object.keys(timetable.days[day]).length !== 0) selectCallback(day); }; })(dayList[d2]);
                var subjectElem = $("<td />").addClass("subject").click(callbackWrapper);
                var roomElem = $("<td />").addClass("room").click(callbackWrapper);
                if (timetable.days[dayList[d2]] && timetable.days[dayList[d2]][periodList[p]])
                {
                    var period = timetable.days[dayList[d2]][periodList[p]];
                    subjectElem.text(edval.classes.getClassInfo(period.classId).name);
                    roomElem.text(period.roomId.toUpperCase());
                }
                else
                {
                    subjectElem.addClass("free");
                    roomElem.addClass("free");
                }
                rowElem.append(subjectElem).append(roomElem);
            }
            wrapperElem.append(rowElem);
        }

        if (table !== tables)
        {
            var headerRowElem = $("<tr />").append($("<th />").addClass("day").addClass("period"));
            for (var d2 = d; d2 < dayList.length * table / tables; ++d2)
                headerRowElem.append($("<th />").attr({colspan: 2}).addClass("day").text(dayNumberToWordMap[dayToDayNumberMap[dayList[d2]]]));
            wrapperElem.append(headerRowElem);
        }
    }
    return wrapperElem;
}

function showTimetableOverview(edval, timetable, selectCallback)
{
    document.body.appendChild(buildTimetableOverview(edval, timetable, selectCallback)[0]);
    updateTimetableOverviewSize();
    if (!localStorage.getItem("TimetableOverview::isFirstRunned"))
    {
        alert("Touch a period for (slightly) more detailed information for that day.");
        localStorage.setItem("TimetableOverview::isFirstRunned", true)
    }
}

function updateTimetableOverviewSize()
{
    var range = document.createRange();
    $(".subject, .room").each(function ()
    {
        if (!this.childNodes[0])
            return;
        range.selectNodeContents(this.childNodes[0]);
        var elem = $(this);
        var scale = elem.width() / range.getBoundingClientRect().width;
        if (scale < 1)
            elem.css({"transform": "scale(" + scale + ", 1)", "transform-origin": "left"});
        else
            elem.css({"transform": "", "transform-origin": ""});
    });
}
window.addEventListener("resize", updateTimetableOverviewSize);
mobile.setMaxScales(1.4, 3);
