package au.edu.nsw.nsbhs.timetable;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileInputStream;

import android.os.Bundle;
import android.os.Build;
import android.app.Activity;
import android.content.Context;

import android.webkit.WebView;
import android.webkit.WebSettings;
import android.webkit.WebChromeClient;
import android.webkit.WebViewClient;

import android.view.Window;
import android.view.KeyEvent;
import android.view.MenuItem;

public class MainActivity extends Activity
{
    private static final String timetableFilename = "timetable.ett";
    private WebView view;

    private Object javaInterface = new Object()
    {
        public void exit() { finish(); }

        public void setTitle(String title)
        {
            setTitle(title);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB)
                getActionBar().setTitle(title);
        }

        public String getTimetableFile()
        {
            try
            {
                FileInputStream in = openFileInput(timetableFilename);
                int length = (int)(new File(getFilesDir().getAbsolutePath() + "/" + timetableFilename).length());
                byte[] buffer = new byte[length];
                int read = 0;
                while (read < length)
                {
                    int r = in.read(buffer, read, length - read);
                    if (r <= 0)
                        break;
                    read += r;
                }
                in.close();
                return new String(buffer);
            }
            catch (java.lang.Exception e)
            {
                return "";
            }
        }
        public void saveTimetableFile(String data)
        {
            try
            {
                FileOutputStream out = openFileOutput(timetableFilename, Context.MODE_PRIVATE);
                out.write(data.getBytes());
                out.close();
            }
            catch (java.lang.Exception e)
            {
                runJavascript("alert(\"Could not save timetable data: " + e.toString() + "\");");
            }
        }
    };

    private void runJavascript(String code) { view.loadUrl("javascript:" + code); }

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        getWindow().requestFeature(Window.FEATURE_PROGRESS);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB)
            setTheme(android.R.style.Theme_Holo_Light);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH)
            getActionBar().setHomeButtonEnabled(true);
        super.onCreate(savedInstanceState);

        view = new WebView(this);
        WebSettings settings = view.getSettings();
        settings.setDefaultZoom(WebSettings.ZoomDensity.FAR);
        settings.setBuiltInZoomControls(false);
        settings.setSupportZoom(false);
        settings.setLoadWithOverviewMode(false);
        settings.setUseWideViewPort(false);
        settings.setJavaScriptEnabled(true);
        settings.setDomStorageEnabled(true);
        settings.setDatabaseEnabled(true);
        settings.setDatabasePath("/data/data/" + getPackageName() + "/databases/");
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN)
            settings.setAllowUniversalAccessFromFileURLs(true);
        view.setWebChromeClient(new WebChromeClient() { @Override public void onProgressChanged(WebView view, int progress) { setProgress(progress * 1000); }});
        view.addJavascriptInterface(javaInterface, "java");
        view.loadUrl("file:///android_asset/index.html");
        setContentView(view);
    }

    @Override protected void onPause() { runJavascript("timetables.saveOpenedTimetables();"); super.onPause(); }
    @Override public void onBackPressed() { runJavascript("tabs.getTab(tabs.getInnermostActiveTabId()).manager.selectParentTab();"); }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event)
    {
        if (keyCode == KeyEvent.KEYCODE_MENU)
        {
            runJavascript("tabs.toggleMenu();");
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        if (item.getItemId() == android.R.id.home)
        {
            runJavascript("tabs.toggleMenu();");
            return true;
        }
        return super.onOptionsItemSelected(item);
    }
}
